#!/usr/bin/env python
# coding: utf-8
#
#       Copyright 2008 Olivier Berten <olivier.berten@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 3 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#

from __future__ import division
import sys
import time
import tempfile

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from swatchbook import *

__version__ = "0.4"

current_sw = False
current_sp = False

swatchbooker_svg = (os.path.dirname(__file__) or ".")+"/icons/swatchbooker.svg"

# 0: float, 1: percentage, 2: degrees
models = SortedDict()
models['Lab'] = (('L',0),('a',0),('b',0))
models['XYZ'] = (('X',0),('Y',0),('Z',0))
models['RGB'] = (('R',1),('G',1),('B',1))
models['CMY'] = (('C',1),('M',1),('Y',1))
models['HLS'] = (('H',2),('L',1),('S',1))
models['HSV'] = (('H',2),('S',1),('V',1))
models['CMYK'] = (('C',1),('M',1),('Y',1),('K',1))
models['GRAY'] = (('K',1),)
models['YIQ'] = (('Y',0),('I',0),('Q',0))

class GroupWidget(QGroupBox):
	def __init__(self, parent=None):
		global current_sw
		super(GroupWidget, self).__init__(parent)
		
		self.setTitle("Group")

		nameLabel = QLabel("Name:")
		self.swName = QLineEdit()
		descriptionLabel = QLabel("Description:")
		self.swDescription = QTextEdit()
		swInfo = QGridLayout()
		swInfo.addWidget(nameLabel, 0, 0)
		swInfo.addWidget(self.swName, 0, 1)
		swInfo.addWidget(descriptionLabel, 1, 0, 1, 2)
		swInfo.addWidget(self.swDescription, 2, 0, 1, 2)
		self.setLayout(swInfo)

		if hasattr(current_sw,'info'):
			if 'name' in current_sw.info:
				self.swName.setText(current_sw.info['name'][0])
			if 'description' in current_sw.info:
				self.swDescription.setText(current_sw.info['description'][0])

		# Actions
		self.connect(self.swName,
				SIGNAL("textEdited(QString)"), self.sw_edit)
		self.connect(self.swDescription,
				SIGNAL("textChanged()"), self.sw_edit)

	def sw_edit(self):
		global current_sw
		if  self.sender() == self.swName:
			if self.swName.text() == '':
				del current_sw.info['name'][0]
			else:
				if 'name' not in current_sw.info:
					current_sw.info['name'] = {}
				current_sw.info['name'][0] = unicode(self.swName.text())
		if  self.sender() == self.swDescription:
			if self.swDescription.toPlainText() == '':
				del current_sw.info['description'][0]
			else:
				if 'description' not in current_sw.info:
					current_sw.info['description'] = {}
				current_sw.info['description'][0] = unicode(self.swDescription.toPlainText())

class ColorWidget(QGroupBox):
	def __init__(self, parent=None):
		super(ColorWidget, self).__init__(parent)
		
		self.parent = parent
		self.setTitle("Color")

		nameLabel = QLabel("Name:")
		self.swName = QLineEdit()
		descriptionLabel = QLabel("Description:")
		self.swDescription = QTextEdit()
		self.sample = QLabel()
		self.sample.setMinimumHeight(30)
		self.swSpot = QCheckBox("Spot")
		self.swValues = QTabWidget()
		self.butVal = QToolButton(self.swValues)
		self.butVal.setFixedSize(12,12)
		cornLay = QVBoxLayout()
		cornLay.setContentsMargins(20,0,0,20)
		cornLay.addWidget(self.butVal)
		cornWid = QWidget()
		cornWid.setLayout(cornLay)
		cornWid.setMinimumSize(12,12)
		self.menuVal = QMenu()
		self.menuValModl = self.menuVal.addMenu('Add')
		global models
		for model in models:
			self.menuValModl.addAction(model,self.addVal)
		self.delValAction = self.menuVal.addAction('Remove',self.delVal)
		self.delValAction.setEnabled(False)
		self.butVal.setPopupMode(QToolButton.InstantPopup)
		self.butVal.setMenu(self.menuVal)
		self.swValues.setCornerWidget(cornWid)
		swInfo = QGridLayout()
		swInfo.addWidget(nameLabel, 0, 0)
		swInfo.addWidget(self.swName, 0, 1)
		swInfo.addWidget(descriptionLabel, 1, 0, 1, 2)
		swInfo.addWidget(self.swDescription, 2, 0, 1, 2)
		swInfo.addWidget(self.sample, 3, 0, 1, 2)
		swInfo.addWidget(self.swSpot, 4, 0, 1, 2)
		swInfo.addWidget(self.swValues, 5, 0, 1, 2)
		self.setLayout(swInfo)

		if hasattr(current_sw,'info'):
			if 'name' in current_sw.info:
				self.swName.setText(current_sw.info['name'][0])
			if 'description' in current_sw.info:
				self.swDescription.setText(current_sw.info['description'][0])
		if hasattr(current_sw,'attr') and 'spot' in current_sw.attr:
			self.swSpot.setChecked(True)
		self.val = {}
		if hasattr(current_sw,'values') and len(current_sw.values) > 0:
			self.delValAction.setEnabled(True)
			settings = QSettings()
			prof_out = settings.value("DisplayProfile").toString() or False
			r,g,b = current_sw.toRGB8(prof_out)
			self.sample.setStyleSheet("QWidget { background-color: rgb("+str(r)+","+str(g)+","+str(b)+") }")
			for model in current_sw.values:
				self.add_val_tab(model,current_sw.values[model])

			self.def_current_sp()

		# Actions
		self.connect(self.swName,
				SIGNAL("textEdited(QString)"), self.sw_edit)
		self.connect(self.swDescription,
				SIGNAL("textChanged()"), self.sw_edit)
		self.connect(self.swSpot,
				SIGNAL("stateChanged(int)"), self.sw_edit)
		self.connect(self.swValues,
				SIGNAL("currentChanged(int)"), self.def_current_sp)

	def add_val_tab(self,model,values=None):
		global current_sw
		global models
		profile = model[1]
		modell = model[0]
		swColor = QWidget()
		grid = QGridLayout()
		width = 2
		count = 0
		if modell in models:
			for elem in models[modell]:
				val = QLineEdit()
				self.val[val] = (model,count)
				self.connect(val,
						SIGNAL("textEdited(QString)"), self.sw_valedit)
				grid.addWidget(QLabel(elem[0]+":"), count, 0)
				if elem[1] == 0:
					if values:
						val.setText(str(round(values[count],2)))
					else:
						val.setText('0')
					grid.addWidget(val, count, 1)
				elif elem[1] == 1:
					width = 3
					if values:
						val.setText(str(round(values[count]*100,2)))
					else:
						val.setText('0')
					grid.addWidget(val, count, 1)
					grid.addWidget(QLabel("%"), count, 2)
				if elem[1] == 2:
					width = 3
					if values:
						val.setText(str(round(values[count]*360,2)))
					else:
						val.setText('0')
					grid.addWidget(val, count, 1)
					grid.addWidget(QLabel(u"°"), count, 2)
				count += 1
		else:
			self.val[model] = {}
			for ink in values:
				val = QLineEdit()
				self.val[val] = (model,count)
				if values:
					val.setText(str(round(values[count],2)))
				else:
					val.setText('0')
				grid.addWidget(val, count, 1)
				count += 1
				
		if modell not in ('Lab','XYZ'):
			grid.addWidget(QLabel("Profile"), count, 0, 1, width)
			if modell in ('RGB','HSL','HSV','YIQ','CMY'):
				modellist = 'RGB'
			else:
				modellist = modell
			profList = QComboBox()
			profList.addItems(self.getProfList(modell))
			if profile in self.parent.sb.profiles:
				profList.setCurrentIndex(self.parent.profiles[modell].index(profile)+1)
			self.connect(profList,
					SIGNAL("currentIndexChanged(int)"), self.change_profile)
			grid.addWidget(profList, count+1, 0, 1, width)
		swColor.setLayout(grid)
		
		self.swValues.addTab(swColor,modell)

	def sw_edit(self):
		global current_sw
		if self.sender() == self.swName:
			if self.swName.text() == '':
				del current_sw.info['name'][0]
				self.parent.itemList[current_sw].setToolTip('')
				self.parent.itemTree[current_sw].setText(0,'')
			else:
				if 'name' not in current_sw.info:
					current_sw.info['name'] = {}
				current_sw.info['name'][0] = unicode(self.swName.text())
				self.parent.itemList[current_sw].setToolTip(self.swName.text())
				self.parent.itemTree[current_sw].setText(0,self.swName.text())
		if self.sender() == self.swDescription:
			if self.swDescription.toPlainText() == '':
				del current_sw.info['description'][0]
			else:
				if 'description' not in current_sw.info:
					current_sw.info['description'] = {}
				current_sw.info['description'][0] = unicode(self.swDescription.toPlainText())
		if self.sender() == self.swSpot:
			if self.swSpot.isChecked():
				current_sw.attr.append('spot')
			else:
				current_sw.attr.remove('spot')
			self.set_preview()

	def set_preview(self):
		settings = QSettings()
		prof_in = settings.value("DisplayProfile").toString() or False
		prof_out = settings.value("CMYKProfile").toString() or False
		if current_sw.toRGB8(prof_out):
			r,g,b = current_sw.toRGB8(prof_out)
			self.sample.setStyleSheet("QWidget { background-color: rgb("+str(r)+","+str(g)+","+str(b)+") }")
			icon = self.parent.colorswatch(current_sw)
		else:
			self.sample.setStyleSheet("")
			icon = QIcon(swatchbooker_svg)
		self.parent.itemTree[current_sw].setIcon(0,icon)
		self.parent.itemList[current_sw].setIcon(icon)

	def sw_valedit(self):
		global models
		global current_sw
		sender = self.val[self.sender()]
		profile = sender[0][1]
		model = sender[0][0]
		if model in models:
			if models[model][sender[1]][1] == 0:
				current_sw.values[sender[0]][sender[1]] = eval(str(self.sender().text()))
			elif models[model][sender[1]][1] == 1:
				current_sw.values[sender[0]][sender[1]] = eval(str(self.sender().text()))/100
			elif models[model][sender[1]][1] == 2:
				current_sw.values[sender[0]][sender[1]] = eval(str(self.sender().text()))/360
		else:
			current_sw.values[sender[0]][sender[1]] = eval(str(self.sender().text()))
		self.set_preview()

	def def_current_sp(self):
		global current_sp
		if self.swValues.count() > 0:
			model = str(self.swValues.tabText(self.swValues.currentIndex()))
			combo = self.swValues.currentWidget().findChild(QComboBox)
			if combo and combo.currentIndex() > 0:
				current_sp = (model,self.parent.profiles[model][combo.currentIndex()-1])
			else:
				current_sp = (model,False)
		else:
			current_sp = False

	def change_profile(self):
		global current_sw
		global current_sp
		value = current_sw.values[current_sp]
		del current_sw.values[current_sp]
		self.def_current_sp()
		current_sw.values[current_sp] = value
		fields = self.swValues.currentWidget().findChildren(QLineEdit)
		for field in fields:
			self.val[field] = (current_sp,self.val[field][1])
		self.set_preview()

	def delVal(self):
		global current_sw
		global current_sp
		value = current_sw.values[current_sp]
		del current_sw.values[current_sp]
		self.swValues.removeTab(self.swValues.currentIndex())
		self.set_preview()

	def addVal(self):
		global models
		global current_sw
		model = (str(self.sender().text()),False)
		if not hasattr(self,'val'):
			self.val = {}
		current_sw.values[model] = []
		self.add_val_tab(model)
		for elem in models[model[0]]:
			current_sw.values[model].append(0)
		self.swValues.setCurrentIndex(self.swValues.count()-1)
		self.def_current_sp()
		self.delValAction.setEnabled(True)
		self.set_preview()
		

	def getProfList(self,model):
		profList = QStringList()
		profList << ''
		if model in self.parent.profiles:
			for prof in self.parent.profiles[model]:
				profList << self.parent.sb.profiles[prof].info['desc'][0]
		return profList

class MainWindow(QMainWindow):

	def __init__(self, file=False, parent=None):
		super(MainWindow, self).__init__(parent)
		
		self.setWindowTitle('SwatchBooker')
		self.setWindowIcon(QIcon(swatchbooker_svg))
		
		self.filename = False
		self.codec = False

		self.sbWidget = QSplitter(Qt.Horizontal)
		# sbInfo
		nameLabel = QLabel("Name:")
		self.sbName = QLineEdit()
		descriptionLabel = QLabel("Description:")
		self.sbDescription = QTextEdit()
		copyrightLabel = QLabel("Copyright:")
		self.copyright = QLineEdit()
		versionLabel = QLabel("Version:")
		self.version = QLineEdit()
		licenseLabel = QLabel("License:")
		self.sbLicense = QTextEdit()
		
		groupBoxInfo1 = QGroupBox("Information")
		sbInfo1 = QGridLayout()
		sbInfo1.addWidget(nameLabel, 0, 0)
		sbInfo1.addWidget(self.sbName, 0, 1)
		sbInfo1.addWidget(descriptionLabel, 1, 0, 1, 2)
		sbInfo1.addWidget(self.sbDescription, 2, 0, 1, 2)
		sbInfo1.addWidget(copyrightLabel, 3, 0)
		sbInfo1.addWidget(self.copyright, 3, 1)
		sbInfo1.addWidget(versionLabel, 4, 0)
		sbInfo1.addWidget(self.version, 4, 1)
		sbInfo1.addWidget(licenseLabel, 5, 0, 1, 2)
		sbInfo1.addWidget(self.sbLicense, 6, 0, 1, 2)


		self.sbProfiles = QTableWidget()
		self.sbProfiles.horizontalHeader().setStretchLastSection(True)
		self.sbProfiles.verticalHeader().hide()
		self.sbProfiles.horizontalHeader().hide()
		self.sbProfiles.setColumnCount(2)
		self.sbProfiles.setColumnHidden(1,True)
		self.sbProfiles.setEditTriggers(QAbstractItemView.NoEditTriggers)
		self.butProf = QToolButton(self)
		self.butProf.setFixedSize(12,12)
		self.menuProf = QMenu()
		self.menuProf.addAction('Add',self.addProfile)
		#self.profEditAction = self.menuProf.addAction('Edit')
		self.profRemoveAction = self.menuProf.addAction('Remove',self.remProfile)
		self.butProf.setPopupMode(QToolButton.InstantPopup)
		self.butProf.setMenu(self.menuProf)
		#self.profEditAction.setEnabled(False)
		self.profRemoveAction.setEnabled(False)

		groupBoxInfo2 = QGroupBox("Color profiles")
		sbInfo2 = QHBoxLayout()
		sbInfo2.addWidget(self.sbProfiles)
		sbInfo2.addWidget(self.butProf,0,Qt.AlignTop)
		

		sbInfo = QVBoxLayout()
		sbInfo.setContentsMargins(0,0,0,0)
		groupBoxInfo1.setLayout(sbInfo1)
		groupBoxInfo2.setLayout(sbInfo2)
		sbInfo.addWidget(groupBoxInfo1)
		sbInfo.addWidget(groupBoxInfo2)
		sbInfoWidget = QWidget()
		sbInfoWidget.setLayout(sbInfo)
		
		self.sbWidget.addWidget(sbInfoWidget)

		# sbTree
		self.treeWidget = QTreeWidget()
		self.treeWidget.setHeaderHidden(True)
		self.treeWidget.setItemsExpandable(True)
		self.swnbLabel = QLabel()
		self.swEditBut = QToolButton(self)
		self.swEditBut.setMaximumSize(12,12)
		self.swEditMenu = QMenu()
		self.swEditMenu.addAction('Add Color',self.swAddColor)
		self.swEditMenu.addAction('Add Gradient',self.swAddGradient)
		self.swEditMenu.addAction('Add Pattern',self.swAddPattern)
		self.swEditMenu.addAction('Add Texture',self.swAddTexture)
		self.swEditMenu.addAction('Add Spacer',self.swAddSpacer)
		self.swEditMenu.addAction('Add Break',self.swAddBreak)
		self.deleteAction = self.swEditMenu.addAction('Delete',self.swDelete)
		self.swEditBut.setPopupMode(QToolButton.InstantPopup)
		self.swEditBut.setMenu(self.swEditMenu)
		self.deleteAction.setEnabled(False)
		
		groupBoxTree = QGroupBox("Tree view")
		sbTree = QGridLayout()
		sbTree.addWidget(self.treeWidget,0,0,1,2)
		sbTree.addWidget(self.swnbLabel,1,0)
		sbTree.addWidget(self.swEditBut,1,1)
		groupBoxTree.setLayout(sbTree)
		self.sbWidget.addWidget(groupBoxTree)

		# sbGrid
		self.listWidget = QListWidget()
		self.listWidget.setViewMode(QListView.IconMode)
		self.listWidget.setMovement(QListView.Static)
		colsLabel = QLabel("Columns:")
		self.cols = QSpinBox()
		self.cols.setRange(0, 64)
		rowsLabel = QLabel("Rows:")
		self.rows = QSpinBox()
		self.rows.setRange(0, 64)

		groupBoxGrid = QGroupBox("Grid view")
		sbGrid = QVBoxLayout()
		dims = QGridLayout()
		dims.addWidget(colsLabel, 0, 0)
		dims.addWidget(self.cols, 0, 1)
		dims.addWidget(rowsLabel, 1, 0)
		dims.addWidget(self.rows, 1, 1)
		dimsWidget = QWidget()
		dimsWidget.setLayout(dims)
		sbGrid.addWidget(self.listWidget)
		sbGrid.addStretch()
		sbGrid.addWidget(dimsWidget)
		groupBoxGrid.setLayout(sbGrid)
		self.sbWidget.addWidget(groupBoxGrid)

		self.setCentralWidget(self.sbWidget)
		
		fileMenu = self.menuBar().addMenu("&File")
		fileMenu.addAction("&New...", self.fileNew, QKeySequence.New)
		fileMenu.addAction("&Open...", self.fileOpen, QKeySequence.Open)
		fileMenu.addAction("&Save As...", self.fileSaveAs, QKeySequence.Save)
		self.menuBar().addAction("Settings", self.settings)
		self.menuBar().addAction("&About", self.about)

		# Actions
		self.connect(self.sbName,
				SIGNAL("textEdited(QString)"), self.sb_edit)
		self.connect(self.sbDescription,
				SIGNAL("textChanged()"), self.sb_edit)
		self.connect(self.copyright,
				SIGNAL("textEdited(QString)"), self.sb_edit)
		self.connect(self.sbLicense,
				SIGNAL("textChanged()"), self.sb_edit)
		self.connect(self.version,
				SIGNAL("textEdited(QString)"), self.sb_edit)
		self.connect(self.cols,
				SIGNAL("valueChanged(int)"), self.sb_edit)
		self.connect(self.rows,
				SIGNAL("valueChanged(int)"), self.sb_edit)
		self.connect(self.treeWidget,
				SIGNAL("itemSelectionChanged()"), self.sw_display_tree)
		self.connect(self.listWidget,
				SIGNAL("itemSelectionChanged()"), self.sw_display_list)
		self.connect(self.sbProfiles,
				SIGNAL("itemSelectionChanged()"),self.prof_editable)

		#Initialisation
		self.sb = SwatchBook()
		if file:
			self.loadFile(file)
		else:
			self.sb_flush()

	def sw_display_tree(self):
		global current_sw
		if self.treeWidget.selectedItems():
			treeItem = self.treeWidget.selectedItems()[0]
			current_sw = item = self.treeItems[treeItem]
			if item.__class__.__name__ not in ('Group','Break'):
				self.listWidget.setCurrentItem(self.itemList[item])
			else:
				self.listWidget.setCurrentItem(None)
			if hasattr(self,'sbSwatch'):
				self.sbSwatch.setParent(None)
			if isinstance(item,Color):
				self.sbSwatch = ColorWidget(self)
			elif isinstance(item,Group):
				self.sbSwatch = GroupWidget()
			if not isinstance(item, Spacer) and not isinstance(item, Break):
				self.sbWidget.addWidget(self.sbSwatch)
			self.deleteAction.setEnabled(True)

	def sw_display_list(self):
		global current_sw
		if self.listWidget.selectedItems():
			listItem = self.listWidget.selectedItems()[0]
			current_sw = item = self.listItems[listItem]
			self.treeWidget.setCurrentItem(self.itemTree[item])

	def sb_edit(self):
		if self.sbName.text() > '':
			if 'name' not in self.sb.info:
				self.sb.info['name'] = {}
			self.sb.info['name'][0] = unicode(self.sbName.text())
		if self.sbDescription.toPlainText() > '':
			if 'description' not in self.sb.info:
				self.sb.info['description'] = {}
			self.sb.info['description'][0] = unicode(self.sbDescription.toPlainText())
		if self.copyright.text() > '':
			if 'copyright' not in self.sb.info:
				self.sb.info['copyright'] = {}
			self.sb.info['copyright'][0] = unicode(self.copyright.text())
		if self.sbLicense.toPlainText() > '':
			if 'license' not in self.sb.info:
				self.sb.info['license'] = {}
			self.sb.info['license'][0] = unicode(self.sbLicense.toPlainText())
		if self.version.text() > '':
			self.sb.info['version'] = unicode(self.version.text())
		if self.cols.value() > 0:
			self.sb.display['columns'] = self.cols.value()
			self.listWidget.setFixedWidth(self.sb.display['columns']*17 + 20)
		elif self.cols.value() == 0:
			self.listWidget.setMinimumWidth(0)
			self.listWidget.setMaximumWidth(0xFFFFFF)
		if self.rows.value() > 0:
			self.sb.display['rows'] = self.rows.value()
			self.listWidget.setFixedHeight(self.sb.display['rows']*17 + 5)
		elif self.rows.value() == 0:
			self.listWidget.setMinimumHeight(0)
			self.listWidget.setMaximumHeight(0xFFFFFF)

	def fileNew(self):
		self.sb_flush()

	def fileOpen(self):
		dir = os.path.dirname(self.filename) \
				if self.filename else "."
		import swatchbook.codecs as codecs
		filetypes = []
		for codec in codecs.reads:
			codec_exts = []
			for ext in eval('codecs.'+codec).ext:
				codec_exts.append('*.'+ext)
			codec_txt = eval('codecs.'+codec).__doc__ +' ('+" ".join(codec_exts)+')'
			filetypes.append(codec_txt)
		allexts = ["*.%s" % unicode(format).lower() \
				   for format in codecs.readexts.keys()]
		filetype = QString()
		fname = unicode(QFileDialog.getOpenFileName(self,
							"SwatchBooker - Choose file", dir,
							("All supported files (%s)" % " ".join(allexts))+";;"+(";;".join(sorted(filetypes)))))
		if fname:
			self.loadFile(fname)

	def sb_flush(self):
		self.filename = False
		self.sbName.clear()
		self.sbDescription.clear()
		self.copyright.clear()
		self.version.clear()
		self.sbLicense.clear()
		self.cols.setValue(0)
		self.cols.clear()
		self.rows.setValue(0)
		self.rows.clear()
		self.treeWidget.clear()
		self.listWidget.clear()
		self.listWidget.setGridSize(QSize(17,17))
		self.listWidget.setResizeMode(QListView.Adjust)
		self.listWidget.setMinimumSize(QSize(0,0))
		self.listWidget.setMaximumSize(QSize(0xFFFFFF,0xFFFFFF))
		self.treeItems = {}
		self.itemTree = {}
		self.listItems = {}
		self.itemList = {}
		self.profiles = {}
		self.swnb = 0
		self.swnbLabel.clear()
		self.deleteAction.setEnabled(False)
		if hasattr(self,'sbSwatch'):
			self.sbSwatch.setParent(None)
		current_sw = False
		current_sp = False

	def loadFile(self, fname=None):
		if fname is None:
			action = self.sender()
			if isinstance(action, QAction):
				fname = unicode(action.data().toString())
				if not self.okToContinue():
					return
			else:
				return
		if fname:
			self.sb_flush()
			self.sb = SwatchBook(fname)
			self.filename = fname
			if 'name' in self.sb.info:
				self.sbName.setText(self.sb.info['name'][0])
			if 'description' in self.sb.info:
				self.sbDescription.setText(self.sb.info['description'][0])
			if 'copyright' in self.sb.info:
				self.copyright.setText(self.sb.info['copyright'][0])
			if 'license' in self.sb.info:
				self.sbLicense.setText(self.sb.info['license'][0])
			if 'version' in self.sb.info:
				self.version.setText(self.sb.info['version'])
			if 'columns' in self.sb.display:
				self.cols.setValue(self.sb.display['columns'])
				self.listWidget.setFixedWidth(self.sb.display['columns']*17 + 20)
			if 'rows' in self.sb.display:
				self.rows.setValue(self.sb.display['rows'])
				self.listWidget.setFixedHeight(self.sb.display['rows']*17 + 5)
			self.sbProfiles.setRowCount(len(self.sb.profiles))
			row = 0
			for prof in self.sb.profiles:
				profItemID = QTableWidgetItem(prof)
				profItemTitle = QTableWidgetItem(self.sb.profiles[prof].info['desc'][0])
				cprt = self.sb.profiles[prof].info['cprt']
				if 0 in cprt:
					profItemTitle.setToolTip(cprt[0])
				elif 'en_US' in cprt:
					profItemTitle.setToolTip(cprt['en_US'])
				self.sbProfiles.setItem(row, 0, profItemTitle)
				self.sbProfiles.setItem(row, 1, profItemID)
				space = self.sb.profiles[prof].info['space'].strip()
				if space in self.profiles:
					self.profiles[space].append(prof)
				else:
					self.profiles[space] = [prof]
				row += 1
			self.populateTree()

	def fileSave(self,codec):
		if self.filename:
			self.sb.write(codec,self.filename)
		else:
			self.fileSaveAs()

	def fileSaveAs(self):
		import swatchbook.codecs as codecs
		filetypes = {}
		for codec in codecs.writes:
			codec_exts = []
			for ext in eval('codecs.'+codec).ext:
				codec_exts.append('*.'+ext)
			codec_txt = eval('codecs.'+codec).__doc__ +' ('+" ".join(codec_exts)+')'
			filetypes[codec_txt] = (codec,eval('codecs.'+codec).ext[0])
		fname = self.filename or "."
		filetype = QString()
		fname = unicode(QFileDialog.getSaveFileName(self,
						"SwatchBooker - Save file", fname,
						";;".join(filetypes.keys()),filetype))
		if fname:
			if fname.rsplit(".",1)[1] != filetypes[unicode(filetype)][1]:
				fname += "."+filetypes[unicode(filetype)][1]
			self.filename = fname
			self.fileSave(filetypes[unicode(filetype)][0])

	def populateTree(self):
		self.fillTree(self.sb.items)
		self.treeWidget.resizeColumnToContents(0)
		self.treeWidget.resizeColumnToContents(1)
		swnbLabelText = str(self.swnb)+' swatch'
		if self.swnb > 1:
			swnbLabelText += 'es'
		self.swnbLabel.setText(swnbLabelText)

	def fillTree(self,items,group = False):
		for item in items.values():
			if group:
				parent = group
			else:
				parent = self.treeWidget
			if isinstance(item,Group):
				treeItem = QTreeWidgetItem(parent,[QString(item.info['name'][0])])
				self.fillTree(item.items,treeItem)
			elif isinstance(item,Spacer):
				listItem = QListWidgetItem(self.listWidget)
				listItem.setIcon(QIcon())
				self.listItems[listItem] = item
				self.itemList[item] = listItem
				treeItem = QTreeWidgetItem(parent,[QString('<spacer>')])
				font = QFont()
				font.setItalic(True)
				treeItem.setFont(0,font)
				treeItem.setTextColor(0,QColor(128,128,128))
			elif isinstance(item,Break):
				treeItem = QTreeWidgetItem(parent,[QString('<break>')])
				font = QFont()
				font.setItalic(True)
				treeItem.setFont(0,font)
				treeItem.setTextColor(0,QColor(128,128,128))
			else:
				listItem = QListWidgetItem(self.listWidget)
				if isinstance(item,Color) and len(item.values) > 0:
					icon = self.colorswatch(item)
				else:
					icon = QIcon(swatchbooker_svg)
				listItem.setIcon(icon)
				self.listItems[listItem] = item
				self.itemList[item] = listItem
				if 'name' in item.info:
					treeItem = QTreeWidgetItem(parent,[QString(item.info['name'][0])])
					listItem.setToolTip(item.info['name'][0])
				else:
					treeItem = QTreeWidgetItem(parent)
				treeItem.setIcon(0,icon)
			self.treeItems[treeItem] = item
			self.itemTree[item] = treeItem
			if group:
				self.treeWidget.expandItem(parent)
			if not isinstance(item, Group) and not isinstance(item, Spacer) and not isinstance(item, Break):
				self.swnb += 1

	def colorswatch(self,swatch):
		settings = QSettings()
		prof_out = settings.value("DisplayProfile").toString() or False
		r,g,b = swatch.toRGB8(prof_out)
		icon = QPixmap(16,16)
		paint = QPainter()
		paint.begin(icon)
		if 'spot' in swatch.attr:
			paint.setBrush(QColor(255,255,255))
			paint.drawRect(-1, -1, 17, 17)
			paint.setBrush(QColor(r,g,b))
			paint.drawEllipse(0, 0, 15, 15)
		else:
			paint.setBrush(QColor(r,g,b))
			paint.drawRect(0, 0, 15, 15)
		paint.end()
		return QIcon(icon)

	def about(self):
		QMessageBox.about(self, "About SwatchBooker",
                """<b>SwatchBooker</b> %s
                <p>&copy; 2008 Olivier Berten
                <p>Qt %s - PyQt %s""" % (
                __version__, QT_VERSION_STR, PYQT_VERSION_STR))

	def settings(self):
		dialog = SettingsDlg(self)
		settings = QSettings()
		if dialog.exec_():
			if dialog.returnDisProf():
				settings.setValue("DisplayProfile",QVariant(dialog.returnDisProf()))
			else:
				settings.remove("DisplayProfile")
			if dialog.returnCMYKProf():
				settings.setValue("CMYKProfile",QVariant(dialog.returnCMYKProf()))
			else:
				settings.remove("CMYKProfile")
				

	def swDelete(self):
		global current_sw
		sw_tbd = current_sw
		if hasattr(self,'sbSwatch'):
			self.sbSwatch.setParent(None)
		tbd = self.get_parent(sw_tbd)
		if isinstance(sw_tbd, Group):
			self.del_group_from_list(tbd[0],tbd[1])
		elif isinstance(sw_tbd, Break):
			pass
		else:
			self.listWidget.takeItem(self.listWidget.row(self.itemList[sw_tbd]))
			del self.itemList[sw_tbd]
			self.swnb -= 1
		if self.itemTree[sw_tbd].parent() == None:
			self.treeWidget.takeTopLevelItem(self.treeWidget.indexOfTopLevelItem(self.itemTree[sw_tbd]))
		else:
			self.itemTree[sw_tbd].parent().takeChild(self.itemTree[sw_tbd].parent().indexOfChild(self.itemTree[sw_tbd]))
		del tbd[0].items[tbd[1]]
		del self.sb.ids[tbd[1]]
		del self.itemTree[sw_tbd]
		swnbLabelText = str(self.swnb)+' swatch'
		if self.swnb > 1:
			swnbLabelText += 'es'
		self.swnbLabel.setText(swnbLabelText)
		current_sw = False
		self.deleteAction.setEnabled(False)
	
	def del_group_from_list(self,parent,group):
		for sw in parent.items[group].items:
			if isinstance(parent.items[group].items[sw], Group):
				self.del_group_from_list(parent.items[group],sw)
			else:
				self.listWidget.takeItem(self.listWidget.row(self.itemList[parent.items[group].items[sw]]))
				del self.itemList[parent.items[group].items[sw]]
				self.swnb -= 1
			del self.sb.ids[sw]
	
	def get_parent(self,value):
		for item in self.sb.ids:
			if self.sb.ids[item][0] == value:
				return (self.sb.ids[item][1],item)
		return False

	def listitemforadd(self,item):
		treeItem = self.itemTree[item]
		if item.__class__.__name__ == 'Group':
			lastchild = treeItem.child(treeItem.childCount()-1)
			return self.treeItems[lastchild]
		elif item.__class__.__name__ == 'Break':
			if treeItem.parent() == None:
				index = self.treeWidget.indexOfTopLevelItem(treeItem)
				return self.treeItems[self.treeWidget.topLevelItem(index-1)]
			else:
				index = treeItem.parent().indexOfChild(treeItem)
				return self.treeItems[treeItem.parent().child(index-1)]
		else:
			return item

	def swAddColor(self):
		global current_sw
		item = Color(self.sb)
		key = 'col'+str(int(time.mktime(time.gmtime())))
		if key in self.sb.ids:
			#sys.stderr.write('duplicate id ['+key+']\n')
			key = key+str(item)
		icon = QIcon(swatchbooker_svg)
		listItem = QListWidgetItem()
		listItem.setIcon(icon)
		self.listItems[listItem] = item
		self.itemList[item] = listItem
		treeItem = QTreeWidgetItem()
		treeItem.setIcon(0,icon)
		self.treeItems[treeItem] = item
		self.itemTree[item] = treeItem
		if self.treeWidget.selectedItems():
			selItem = self.get_parent(current_sw)
			index = selItem[0].items.values().index(current_sw)
			selItem[0].items.insert(index+1,key,item)
			self.sb.ids[key] = (item,selItem[0])
			selTItem = self.treeWidget.selectedItems()[0]
			if selTItem.parent() == None:
				tIndex = self.treeWidget.indexOfTopLevelItem(selTItem)
				self.treeWidget.insertTopLevelItem(tIndex+1, treeItem)
			else:
				tIndex = selTItem.parent().indexOfChild(selTItem)
				selTItem.parent().insertChild(tIndex+1,treeItem)
			if current_sw.__class__.__name__ not in ('Group','Break'):
				selLItem = self.listWidget.selectedItems()[0]
			else:
				nitem = self.treeItems[selTItem]
				while nitem.__class__.__name__ in ('Group','Break'):
					nitem = self.listitemforadd(nitem)
				selLItem = self.itemList[nitem]
			lIndex = self.listWidget.indexFromItem(selLItem).row()
			self.listWidget.insertItem(lIndex+1,self.itemList[item])
		else:
			self.sb.items[key] = item
			self.sb.ids[key] = (item,self.sb)
			self.treeWidget.addTopLevelItem(self.itemTree[item])
			self.listWidget.addItem(self.itemList[item])
		self.treeWidget.setCurrentItem(self.itemTree[item])
		self.swnb += 1
		swnbLabelText = str(self.swnb)+' swatch'
		if self.swnb > 1:
			swnbLabelText += 'es'
		self.swnbLabel.setText(swnbLabelText)
		current_sw = item

	def swAddBreak(self):
		global current_sw
		item = Break()
		key = 'br'+str(int(time.mktime(time.gmtime())))
		if key in self.sb.ids:
			#sys.stderr.write('duplicate id ['+key+']\n')
			key = key+str(item)
		treeItem = QTreeWidgetItem([QString('<break>')])
		font = QFont()
		font.setItalic(True)
		treeItem.setFont(0,font)
		treeItem.setTextColor(0,QColor(128,128,128))
		self.treeItems[treeItem] = item
		self.itemTree[item] = treeItem
		if self.treeWidget.selectedItems():
			selItem = self.get_parent(current_sw)
			index = selItem[0].items.values().index(current_sw)
			selItem[0].items.insert(index+1,key,item)
			self.sb.ids[key] = (item,selItem[0])
			selTItem = self.treeWidget.selectedItems()[0]
			if selTItem.parent() == None:
				tIndex = self.treeWidget.indexOfTopLevelItem(selTItem)
				self.treeWidget.insertTopLevelItem(tIndex+1, treeItem)
			else:
				tIndex = selTItem.parent().indexOfChild(selTItem)
				selTItem.parent().insertChild(tIndex+1,treeItem)
		else:
			self.sb.items[key] = item
			self.sb.ids[key] = (item,self.sb)
			self.treeWidget.addTopLevelItem(self.itemTree[item])
		self.treeWidget.setCurrentItem(self.itemTree[item])

	def swAddSpacer(self):
		global current_sw
		item = Spacer()
		key = 'sp'+str(int(time.mktime(time.gmtime())))
		if key in self.sb.ids:
			#sys.stderr.write('duplicate id ['+key+']\n')
			key = key+str(item)
		listItem = QListWidgetItem()
		listItem.setIcon(QIcon())
		self.listItems[listItem] = item
		self.itemList[item] = listItem
		treeItem = QTreeWidgetItem([QString('<spacer>')])
		font = QFont()
		font.setItalic(True)
		treeItem.setFont(0,font)
		treeItem.setTextColor(0,QColor(128,128,128))
		self.treeItems[treeItem] = item
		self.itemTree[item] = treeItem
		if self.treeWidget.selectedItems():
			selItem = self.get_parent(current_sw)
			index = selItem[0].items.values().index(current_sw)
			selItem[0].items.insert(index+1,key,item)
			self.sb.ids[key] = (item,selItem[0])
			selTItem = self.treeWidget.selectedItems()[0]
			if selTItem.parent() == None:
				tIndex = self.treeWidget.indexOfTopLevelItem(selTItem)
				self.treeWidget.insertTopLevelItem(tIndex+1, treeItem)
			else:
				tIndex = selTItem.parent().indexOfChild(selTItem)
				selTItem.parent().insertChild(tIndex+1,treeItem)
			if current_sw.__class__.__name__ not in ('Group','Break'):
				selLItem = self.listWidget.selectedItems()[0]
			else:
				nitem = self.treeItems[selTItem]
				while nitem.__class__.__name__ in ('Group','Break'):
					nitem = self.listitemforadd(nitem)
				selLItem = self.itemList[nitem]
			lIndex = self.listWidget.indexFromItem(selLItem).row()
			self.listWidget.insertItem(lIndex+1,self.itemList[item])
		else:
			self.sb.items[key] = item
			self.sb.ids[key] = (item,self.sb)
			self.treeWidget.addTopLevelItem(self.itemTree[item])
		self.treeWidget.setCurrentItem(self.itemTree[item])

	def swAddTexture(self):
		print 'unimplemented'

	swAddGradient = swAddPattern = swAddTexture

	def prof_editable(self):
		if self.sbProfiles.isItemSelected(self.sbProfiles.currentItem()):
			#self.profEditAction.setEnabled(True)
			self.profRemoveAction.setEnabled(True)

	def addProfile(self):
		fname = unicode(QFileDialog.getOpenFileName(self,
							"SwatchBooker - Choose file", ".",
							("ICC profiles (*.icc *.icm)")))
		if fname:
			# the next 6 lines are a workaround for the unability of lcms to deal with unicode file names
			fi = open(fname)
			uri = tempfile.mkstemp()[1]
			fo = open(uri,'w')
			fo.write(fi.read())
			fi.close()
			fo.close()
			import swatchbook.icc as icc
			profile = icc.ICCprofile(uri)
			#TODO: check if exists
			id = os.path.basename(fname)
			self.sb.profiles[id] = profile
			profItemID = QTableWidgetItem(id)
			profItemTitle = QTableWidgetItem(profile.info['desc'][0])
			cprt = profile.info['cprt']
			if 0 in cprt:
				profItemTitle.setToolTip(cprt[0])
			elif 'en_US' in cprt:
				profItemTitle.setToolTip(cprt['en_US'])
			row = self.sbProfiles.rowCount()
			self.sbProfiles.setRowCount(row+1)
			self.sbProfiles.setItem(row, 0, profItemTitle)
			self.sbProfiles.setItem(row, 1, profItemID)
			space = profile.info['space'].strip()
			if space in self.profiles:
				self.profiles[space].append(id)
			else:
				self.profiles[space] = [id]

	def remProfile(self):
		profid = unicode(self.sbProfiles.item(self.sbProfiles.currentItem().row(),1).text())
		self.sbProfiles.removeRow(self.sbProfiles.currentItem().row())
		self.profiles[self.sb.profiles[profid].info['space'].strip()].remove(profid)
		del self.sb.profiles[profid]
		#self.profEditAction.setEnabled(False)
		self.profRemoveAction.setEnabled(False)
		# TODO remove profile from color values

class SettingsDlg(QDialog):
	def __init__(self, parent=None):
		super(SettingsDlg, self).__init__(parent)
		
		buttonBox = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)

		self.sRGB = QCheckBox("sRGB")
		self.RGBfileLabel = QLabel()
		self.RGBfileLabel.setFrameStyle(QFrame.StyledPanel|QFrame.Sunken)
		self.RGBfileButton = QPushButton("Choose file")

		RGBfileLayout = QHBoxLayout()
		RGBfileLayout.addWidget(self.RGBfileLabel, 1)
		RGBfileLayout.addWidget(self.RGBfileButton)
		
		gDisProf = QGroupBox("Display Profile")
		disprof = QVBoxLayout()
		disprof.addWidget(self.sRGB)
		disprof.addLayout(RGBfileLayout)
		gDisProf.setLayout(disprof)

		self.Fogra27L = QCheckBox("Fogra27L")
		self.CMYKfileLabel = QLabel()
		self.CMYKfileLabel.setFrameStyle(QFrame.StyledPanel|QFrame.Sunken)
		self.CMYKfileButton = QPushButton("Choose file")

		CMYKfileLayout = QHBoxLayout()
		CMYKfileLayout.addWidget(self.CMYKfileLabel, 1)
		CMYKfileLayout.addWidget(self.CMYKfileButton)

		gCMYKProf = QGroupBox("Default CMYK Profile")
		CMYKprof = QVBoxLayout()
		CMYKprof.addWidget(self.Fogra27L)
		CMYKprof.addLayout(CMYKfileLayout)
		gCMYKProf.setLayout(CMYKprof)

		sett = QVBoxLayout()
		sett.addWidget(gDisProf)
		sett.addWidget(gCMYKProf)
		sett.addWidget(buttonBox)
		self.setLayout(sett)

		settings = QSettings()
		if settings.contains("DisplayProfile"):
			self.RGBfname = settings.value("DisplayProfile").toString()
			self.RGBfileLabel.setText(self.RGBfname)
		else:
			self.sRGB.setCheckState(2)
			self.RGBfileButton.setEnabled(False)
		if settings.contains("CMYKProfile"):
			self.CMYKfname = settings.value("CMYKProfile").toString()
			self.CMYKfileLabel.setText(self.CMYKfname)
		else:
			self.Fogra27L.setCheckState(2)
			self.CMYKfileButton.setEnabled(False)

		self.connect(buttonBox, SIGNAL("accepted()"), self, SLOT("accept()"))
		self.connect(buttonBox, SIGNAL("rejected()"), self, SLOT("reject()"))
		self.connect(self.sRGB, SIGNAL("stateChanged (int)"), self.actProfile)
		self.connect(self.RGBfileButton, SIGNAL("clicked()"), self.setRGBFile)
		self.connect(self.Fogra27L, SIGNAL("stateChanged (int)"), self.actProfile)
		self.connect(self.CMYKfileButton, SIGNAL("clicked()"), self.setCMYKFile)
		self.setWindowTitle("SwatchBooker - Settings")

	def accept(self):
		if self.sRGB.checkState() == 0 and not hasattr(self,'RGBfname'):
			msgBox = QMessageBox(self)
			msgBox.setWindowTitle('Error')
			msgBox.setIcon(QMessageBox.Critical)
			msgBox.setText("You must choose either sRGB or a file")
			msgBox.exec_()
		elif self.Fogra27L.checkState() == 0 and not hasattr(self,'CMYKfname'):
			msgBox = QMessageBox(self)
			msgBox.setWindowTitle('Error')
			msgBox.setIcon(QMessageBox.Critical)
			msgBox.setText("You must choose either Fogra27L or a file")
			msgBox.exec_()
		else:
			QDialog.accept(self)

	def returnDisProf(self):
		if hasattr(self,'RGBfname'):
			return self.RGBfname

	def returnCMYKProf(self):
		if hasattr(self,'CMYKfname'):
			return self.CMYKfname

	def actProfile(self):
		if self.sRGB.checkState() == 2:
			if hasattr(self,'RGBfname'):
				del self.RGBfname
			self.RGBfileLabel.setText("")
			self.RGBfileButton.setEnabled(False)
		else:
			self.RGBfileButton.setEnabled(True)
		if self.Fogra27L.checkState() == 2:
			if hasattr(self,'CMYKfname'):
				del self.CMYKfname
			self.CMYKfileLabel.setText("")
			self.CMYKfileButton.setEnabled(False)
		else:
			self.CMYKfileButton.setEnabled(True)

	def setRGBFile(self):
		fname = QFileDialog.getOpenFileName(self, "Choose file", QDir.homePath(),"ICC profiles (*.icc *.icm)")
		if fname:
			import swatchbook.icc as icc
			profile = icc.ICCprofile(fname)
			if profile.info['class'] == "mntr" and profile.info['space'] == 'RGB ':
				self.RGBfname = fname
				self.RGBfileLabel.setText(self.RGBfname)
			else:
				msgBox = QMessageBox(self)
				msgBox.setWindowTitle('Error')
				msgBox.setIcon(QMessageBox.Critical)
				msgBox.setText("This isn't a RGB monitor profile")
				msgBox.exec_()
		
	def setCMYKFile(self):
		fname = QFileDialog.getOpenFileName(self, "Choose file", QDir.homePath(),"ICC profiles (*.icc *.icm)")
		if fname:
			import swatchbook.icc as icc
			profile = icc.ICCprofile(fname)
			if profile.info['space'] == 'CMYK':
				self.CMYKfname = fname
				self.CMYKfileLabel.setText(self.CMYKfname)
			else:
				msgBox = QMessageBox(self)
				msgBox.setWindowTitle('Error')
				msgBox.setIcon(QMessageBox.Critical)
				msgBox.setText("This isn't a CMYK profile")
				msgBox.exec_()
		
if __name__ == "__main__":
	app = QApplication(sys.argv)
	app.setOrganizationName("Selapa")
	app.setOrganizationDomain("selapa.net")
	app.setApplicationName("SwatchBooker")
	if len(sys.argv) > 1:
		form = MainWindow(sys.argv[1])
	else:
		form = MainWindow()
	form.show()
	app.exec_()
